import * as mc from "@minecraft/server";
import Vec3 from "../../../modules/Vec3.js";
import range from "../../../modules/arrayRange.js";
import Mathf from '../../../modules/Mathf.js';

export default class stand {
    static sound(system,mode,type,data) {
        let player = system.standMaster
        if (data[type] !== undefined) {
            for (let i in data[type]) {
                let sound = {
                    name: data[type][i].split("->")[0],
                    interval: parseInt(data[type][i].split("->")[1])
                };
                let sound_type = (type) => {
                    let sound_mode = {
                        play: `playsound ${sound.name} @a ~ ~ ~`,
                        stop: `stopsound @a ${sound.name}`
                    }
                    return sound_mode[type]
                }
                switch (mode) {
                    case "play":
                        if (system.fist_duration2.var == 0 || system.fist_duration2.var == 1 || system.fist_duration2.var % sound.interval == 0) {
                            system.runCommand(sound_type("play"), player);
                        }
                        break;
                    case "stop":
                        system.runCommand(sound_type("stop"), player);
                        break;
                }
                
            }
        }
    }
    static fist(system,source,data,fist_data) {
        let lc = system.getLc(source);
        let attackedEntityTag = "isAttackedByFist";
        let attackedProjectileTag = "isHitByFist";
        let stand_range = data.range + system.GlobalVar(mc.world,"stand_range_limit").var;
        
        for (let i of range(2, (stand_range + 1) * 12 * 0.1)) {
            let pos = "xyz".split("").map(pos => {
                let value = Math.round(i * lc[pos]);
                return "~" + value;
            }).join(" ");
            let attack_range = (r,tag) => {
                return `execute positioned ${pos} as @s run tag @e[type=!goldexperience:tree,name=!"${source.name}",r=${r}] add ${tag}`
            };
            
            system.runCommands([
                attack_range(1.12,attackedEntityTag),
                attack_range(1.65,attackedProjectileTag)
            ], source);
            
            system.getEntitiesFromViewVector(source, 1).forEach(entity => {
                entity.addTag(attackedEntityTag);
            })
        }
        
        
        if (fist_data.condition.mental_value) {
            system.removeMentalValue(data.mental.value);
        }
        
        let default_fist_sound = `playsound game.player.attack.strong @a ~ ~ ~`;
        system.runCommand(default_fist_sound, source);
        this.sound(system,fist_data.fist_sound,"fist_sounds",data);
        this.sound(system,fist_data.hit_sound,"hit_sounds",data);
        
        
        if (fist_data.condition.attack_entity) {
            let attackedEntity = system.getEntities({tags:[attackedEntityTag]}).filter(e => (e?.hasComponent("movement") || e?.id == "minecraft:armor_stand" || e?.id == "minecraft:boat" || e?.id == "minecraft:minecart"));
            if (attackedEntity.length <= 0) {
                let block = system.getBlockFromViewVector(source, stand_range + 3, {liquid: false, passable: false});
                if (block !== undefined && block !== null) {
                    let blockPos = block.location;
                    let playerPos = source.location;
                    system.set_stand_range(block,stand_range);
                }
            } else {
                attackedEntity.forEach(entity => {
                    let v = fist_data.repel;
                    let health = entity.getComponent("minecraft:health");
                    let command1a = `damage @s[type=!ender_dragon] ${fist_data.damage} suicide`;
                    let command1b = `damage @s[type=!ender_dragon] ${fist_data.damage} entity_attack entity @a[name="${source.name}"]`;
                    let command2 = `damage @s[type=ender_dragon] ${fist_data.damage} suicide entity @a[name="${source.name}"]`;
                    let command3 = `effect @s[type=!player] slowness 1 10 true`;
                    system.runCommands([
                        health.current <= fist_data.damage ? command1b : command1a,
                        command2,
                        command3
                    ], entity);
                    // console.error(entity.typeId !== "minecraft:player")
                    if (entity.typeId === "minecraft:player") {
                        entity.applyKnockback(lc.x, lc.z, 1.75 * v * system.getLc2(-90,0).y - Math.abs(Math.round(source.getViewDirection().y)), source.getViewDirection().y * v * 1.75);
                    } else {
                        entity.applyImpulse(new Vec3(lc.x * v, lc.y * v, lc.z * v).Vector());
                    }
                    
                    let entityPos = entity.location;
                    let playerPos = source.location;
                    
                    system.set_stand_range(entity,stand_range);
                    
                    entity.removeTag(attackedProjectileTag);
                    entity.removeTag(attackedEntityTag);
                        
                })
                
            }
        }
        if (fist_data.condition.hit_projectile) {
            let projectiles = system.getEntities({tags:[attackedProjectileTag]}).filter(e => !e?.hasComponent("minecraft:movement") && e?.id != "minecraft:armor_stand" && e?.id != "minecraft:boat" && e?.id != "minecraft:minecart" && e?.id != "minecraft:item");
            if (projectiles.length > 0) {
                projectiles.forEach(entity => {
                    let v = Math.floor(Mathf.random(2,4));
                    entity.applyImpulse(new Vec3(lc.x * v + Math.round(Mathf.random(-1,1)), lc.y * v - 0.5, lc.z * v + Math.round(Mathf.random(-1,1))).Vector());
                    entity.removeTag(attackedProjectileTag);
                    entity.removeTag(attackedEntityTag);
                })
            }
        }
    }
}