import * as mc from "@minecraft/server";
import stand from "./other.js";
import titleUI from "./standUI.js";
import Vec3 from "../../../modules/Vec3.js";
import range from "../../../modules/arrayRange.js";
import Mathf from '../../../modules/Mathf.js';
import registerSystem from "../../../register/registerSystem.js";

export default class stand_defaultAttack extends registerSystem {
    property(name) {
        return this.GlobalVar(this.standMaster,name);
    }
    global(name) {
        return this.GlobalVar(this.standMaster,name);
    }
    globalW(name) {
        return this.GlobalVar(mc.world,name);
    }
    constructor(player) {
        super();
        this.standMaster = player;
        
        this.skills_count = 2;
        
        this.health = this.standMaster.getComponent("minecraft:health");
        this.movement = this.standMaster.getComponent("minecraft:movement");
        this.inventory = this.standMaster.getComponent("minecraft:inventory").container;
        
        this.stand_range = this.scores(player,"stand_range");
        
        
        this.fist_duration = this.GlobalVar(this.standMaster,"stand_skill1");
        this.fist_duration2 = this.GlobalVar(this.standMaster,"stand_skill1_2");
        this.mental_value = this.GlobalVar(this.standMaster,"mental_value");
        this.mental_reply = this.GlobalVar(this.standMaster,"mental_reply");
        this.mental_timer = this.GlobalVar(this.standMaster,"mental_timer");
        this.barrage_camera_shake = this.GlobalVar(this.standMaster,"barrage_camera_shake");
        this.mental_infinite = this.GlobalVar(mc.world,"mental.is_infinite");
        this.mental_recovery = this.GlobalVar(mc.world,"mental.can_recovery");
        // this.fist_duration = this.property("stand_skill1");
        // this.fist_duration2 = this.property("stand_skill1_2");
        // this.mental_value = this.property("mental_value");
        // this.mental_reply = this.property("mental_reply");
        // this.mental_timer = this.property("mental_timer");
        // this.barrage_camera_shake = this.global("barrage_camera_shake");
        // this.mental_infinite = this.globalW("mental.is_infinite");
        // this.mental_recovery = this.globalW("mental.can_recovery");
        
        this.stand = /stand:skill1/;
        this.stand2 = /stand:\w+/;
        
        this.check = this.mental_value.var >= 0 && this.mental_value.var <= 70;
        
        this.customUI = new titleUI(this.standMaster);
        this.MentalPanelUI = () => {
            return this.customUI.loadingUI3("@ui.stand.mental->" + (this.mental_infinite.var ? 70 : Mathf.clamp(this.mental_value.var,0,70)));
        }
        this.LoadingGUI = (ui) => {
            return this.customUI.loadingGUI(ui);
        }
        this.LoadingUI2 = (ui) => {
            return this.customUI.loadingUI2(ui);
        }
        this.removeMentalValue = (value) => {
            this.MentalPanelUI();
            this.mental_value.var -= this.mental_infinite.var ? 0 : value;
            this.mental_reply.var = 0;
        }
        this.addMentalValue = (value) => {
            this.MentalPanelUI();
            this.mental_value.var += value;
            this.mental_reply.var = 0;
        }
        /*
        if (this.check && this.mental_reply.var <= (70 - this.mental_value.var + 10) * 15) {// (70-30+10)*10
            this.mental_reply.var += this.mental_recovery.var ? 1 : 0;
        } else if (this.check) {
            this.mental_reply.var = 0;
            this.mental_value.var += 1;
        }*/
        
        
        
        
        this.main();
        
    }
    
    main() {
        let player = this.standMaster;
        
        
        let itemTag = JSON.stringify({
            "minecraft:item_lock": {
                mode: "lock_in_inventory"
            }
        });
        if (this.mental_timer.var == 0 || this.mental_timer.var == 3) {
            this.MentalPanelUI();
        }
        if (this.mental_timer.var == 3) {
            // console.error(range(1,this.skills_count + 1).join(","))
            this.mental_timer.var = 0;
            player?.addEffect(mc.MinecraftEffectTypes["healthBoost"],9999,19,false);
            let skills = range(1,this.skills_count).map(v => {
                return `{item=stand:skill${v},quantity=0}`
            }).join(",");
            this.runCommand(`tag @s[hasitem=[${skills}]] add stand.no_skills`, player);
        }
        if (player?.hasTag("stand.no_skills")) {
            for (let i of range(1,this.skills_count)) {
                let inventory = player?.getComponent("inventory").container;
                let itemStack = new mc.ItemStack(mc.ItemTypes.get("stand:skill" + i));
                itemStack.keepOnDeath = true,
                itemStack.lockMode = "inventory",
                inventory?.addItem(itemStack)
            }
            player?.removeTag("stand.no_skills");
        }
    }
    
    set_stand_range(target,range) {
        let player = this.standMaster;
        let playerPos = player?.location;
        let distance = new Vec3(target.location.x, target.location.y, target.location.z).Distance(playerPos.x, playerPos.y, playerPos.z);
        let value = Math.min(distance.toFixed(2) * 100, range * 100)
        this.stand_range.var = value
    }
    
    broadcastAnim(name) {
        let player = this.standMaster;
        let arrow = "->";
        let startsName = name.split(arrow)[0];
        let endsName = name.split(arrow)[1];
        return this.runCommand(`playanimation @s animation.${startsName}.variable_create.${endsName} default 1`, player);
    }
    removeAnim(name) {
        let player = this.standMaster;
        let arrow = "->";
        let startsName = name.split(arrow)[0];
        let endsName = name.split(arrow)[1];
        return this.runCommand(`playanimation @s animation.${startsName}.variable_remove.${endsName} default 1`, player);
    }
    
    summon(data) {
        let player = this.standMaster;
        let handItem = this.getHandItem(player);
        
        if (this.stand.exec(handItem.typeId) && !player?.hasTag("stand.summon") && this.mental_value.var > 0) {
            data.sounds.forEach(sound => {
                this.runCommand(`playsound ${sound} @a[tag=!player?.isSkippingTime,r=10] ~ ~ ~`, player);
            })
            this.broadcastAnim(`stand->summon.${data.anim_id}`);
            player?.addTag("stand.summon");
        } else if ((!this.stand2.exec(handItem.typeId) && player?.hasTag("stand.summon")) || this.mental_value.var <= 0) {
            this.removeAnim("stand->remove");
            player?.removeTag("stand.summon");
        }
    }
    
    quickFist(data) {
        let tag = "替身连打攻击";
        
        let player = this.standMaster;
        
        let handItem = this.getHandItem(player);
        
        if (!player?.hasTag("stand.hit") && !player?.hasTag("stand.fist_isCooldown") && player?.hasTag("stand.quick_fist") && this.mental_value.var > 0 && this.fist_duration.var > 0 && this.fist_duration.var <= data.duration * 20) {
            this.fist_duration.var -= 1;
        }
        if (!player?.hasTag("stand.hit") && player?.hasTag("stand.fist_isCooldown") && this.mental_value.var > 0 && this.fist_duration.var >= 0 && this.fist_duration.var < data.duration * 20) {
            this.fist_duration.var += 1;
            if (this.fist_duration.var < data.duration * 20) {
                player?.removeTag("stand.quick_fist");
            }
        }
        
        if (this.fist_duration.var >= data.duration * 20) {
            player?.removeTag("stand.fist_isCooldown");
        }
        
        if (!player?.hasTag("stand.hit") && !player?.hasTag("stand.fist_isCooldown") && player?.hasTag("stand.quick_fist") && this.fist_duration.var > 0 && this.mental_value.var > 0) {
            
            player?.removeTag("stand.stop_fist");
            this.fist_duration2.var += 1;
            
            if (this.fist_duration2.var == 1) {
                this.broadcastAnim(`stand->quick_fist`);
                if (this.barrage_camera_shake.var) {
                    this.runCommand(`camerashake add @s 0.5 5`, player)
                }
            }
            stand.fist(this, player, data, {
                repel: 0.1,
                damage: data.damage,
                fist_sound: "play",
                hit_sound: "stop",
                condition: {
                    mental_value: this.fist_duration2.var % data.mental.interval == 0,
                    attack_entity: this.fist_duration2.var == 0 || this.fist_duration2.var == 1 || this.fist_duration2.var % 4 == 0,
                    hit_projectile: this.fist_duration2.var == 0 || this.fist_duration2.var == 1 || this.fist_duration2.var % 1 == 0
                }
            })
        }
        if ((player?.hasTag("stand.quick_fist") && this.fist_duration.var <= 0) || (!player?.hasTag("stand.quick_fist"))) {
            if (!player?.hasTag("stand.hit") && this.fist_duration2.var > 0) {
                this.fist_duration2.var = 0;
                player?.addTag("stand.hit");
                player?.addTag("stand.fist_isCooldown");
                this.runCommand(`camerashake stop @s`, player)
                this.removeAnim("stand->quick_fist");
                stand.fist(this, player, data, {
                    repel: 3,
                    damage: data.damage + 2,
                    fist_sound: "stop",
                    hit_sound: "play",
                    condition: {
                        mental_value: true,
                        attack_entity: true,
                        hit_projectile: true
                    }
                })
            } 
            if (player?.hasTag("stand.hit")) {
                this.fist_duration2.var += 1;
                if (this.fist_duration2.var >= 15) {
                    let attackedEntityTag = "isAttackedByFist";
                    let attackedProjectileTag = "isHitByFist";
                    player?.removeTag("stand.quick_fist");
                    player?.removeTag("stand.hit");
                    this.runCommands([
                        `tag @e[tag=${attackedEntityTag}] remove ${attackedEntityTag}`,
                        `tag @e[tag=${attackedProjectileTag}] remove ${attackedProjectileTag}`
                    ], player);
                    this.fist_duration2.var = 0;
                }
            }
        }
    }
    
}
