import * as mc from '@minecraft/server';
import stand_defaultAttack from '../../default/stand_defaultAttack.js';
// import titleUI from "../../default/standUI.js";
import titleUIList from '../_ui_/titleUIList.js';
import Vec3 from '../../../../modules/Vec3.js';
import range from '../../../../modules/arrayRange.js';
import Mathf from '../../../../modules/Mathf.js';

export default class kingCrimson extends stand_defaultAttack {
    constructor(player) {
        super(player);
        this.standMaster = player;
        this.UI = titleUIList.ui;
        this.GUI = titleUIList.gui;
        
        this.ap_cd = this.GlobalVar(this.standMaster,"stand_skill2");
        this.ap_cd2 = this.GlobalVar(this.standMaster,"stand_skill2_2");
        this.epitaph_tick = this.GlobalVar(this.standMaster,"stand_skill3");
        this.epitaph_tick2 = this.GlobalVar(this.standMaster,"stand_skill3_2");
        this.erase_tick = this.GlobalVar(this.standMaster,"stand_skill4");
        this.erase_tick2 = this.GlobalVar(this.standMaster,"stand_skill4_2");
        this.playerHealth = this.GlobalVar(this.standMaster,"playerHealth");
        this.playerGameMode = this.GlobalVar(this.standMaster,"playerGameMode");
        
        this.ticking();
        // this.getPressurePlateMaterial = new Array("acacia", "birch", "crimson", "dark_oak", "heavy_weighted", "jungle", "light_weighted", "polished_black_stone", "spruce", "stone", "warped", "wooden");
    }
    
    ticking() {
        
        let player = this.standMaster;
        let handItem = this.getHandItem(player);
        
        let erase_cooldown = 3;
        
        this.skills_count = 2;
        
        this.LoadingGUI({
            name: "standCode:D",
            value: [
                (this.fist_duration.var / 20).toFixed(1),
                (this.ap_cd.var / 20).toFixed(1),
                (this.epitaph_tick.var / 20).toFixed(1),
                (this.erase_tick.var / 20).toFixed(1) + (this.standMaster.hasTag("kc.erase_cooldown") ? `§e|§b${(erase_cooldown - this.erase_tick2.var / 20).toFixed(1)}§r` : "§r")
            ]
        });
        this.summon({
            anim_id: "king_crimson",
            sounds: [
                "kingCrimson.name"
            ]
        });
        
        this.quickFist({
            range: 4,
            damage: 5,
            duration: 1.8,
            fist_sounds: [
                "kingCrimson.quick_fist->4"
            ],
            hit_sounds: [
                "kingCrimson.punch->20"
            ],
            mental: {
                value: 1,
                interval: Math.floor(Mathf.random(20,30))
            }
        });
        this.APattack({
            range: 4,
            damage: 20,
            cooldown: 3,
            mental: 1
        });
        
        this.epitaph({
            duration: 4,
            mental: {
                value: Math.floor(Mathf.random(1,3)),
                interval: 40
            }
        });
        
        this.timeErase({
            duration: 10,
            cooldown: erase_cooldown,
            mental: {
                value: Math.floor(Mathf.random(1,3)),
                interval: 40
            }
        });
        
        
    }
    
    static description(player,system) {
        let _this = system;
        let menu = _this.ui();
        menu.layout(player, `
            <ui id="_stand_description_"/>
                <title text="ui.stand_settings.skills_description"/>
                    <body text="ui.how_to_use.no_stand"/>
                        <button id="skill1" text="ui.stand_settings.3.how_to_use.stand_fist" icon="textures/gui/kingCrimson/fist"/>
                        <button id="skill2" text="ui.stand_settings.3.how_to_use.stand_ap" icon="textures/gui/kingCrimson/ap"/>
                        <button id="skill3" text="ui.stand_settings.3.how_to_use.kingCrimson_epitaph" icon="textures/gui/kingCrimson/epitaph"/>
                        <button id="skill4" text="ui.stand_settings.3.how_to_use.kingCrimson_timeErase" icon="textures/gui/kingCrimson/time_erase"/>
                        <button id="close" text="ui.stand_settings.3"/>
                    </body>
                </title>
            </ui>
        `
        );
        menu.trigger("_stand_description_", e => {
            let canceled = e.canceled;
            _this.ui_using(player,0);
            
            let selection = e.getElementById;
            _this.menu_op(player);
        });
    }
    
    static eventData(_this) {
        let c = (entity) => {
            return entity !== undefined && entity.typeId == "minecraft:player" && entity.hasTag("standMaster.kingCrimson")
        }
        let itemUseInEraseTime = (eventData) => {
            const { source, item } = eventData;
            if (c(source)) {
                let player = source;
                if (player?.hasTag("kingCrimson.timeErase")) {
                    _this.runCommands([
                        `tag @s remove kingCrimson.timeErase`
                    ], player);
                }
            }
        }
        
        _this.listenForEvent("minecraft:itemUse", (eventData) => itemUseInEraseTime(eventData));
        _this.listenForEvent("minecraft:itemUseOn", (eventData) => itemUseInEraseTime(eventData));
        _this.listenForEvent("minecraft:entityHit", (eventData) => {
            let { entity, hitEntity, hitBlock } = eventData;/*
            if (c(entity)) {
                let player = entity;
                let target = hitEntity;
                let block = hitBlock;
                let handItem = _this.getHandItem(player);
                if (/stand:skill1/.exec(handItem.typeId) && player?.hasTag("isSneaking")) {
                    _this.runCommand(`tag @s add stand.APattack`, player);
                    _this.runCommand(`tag @s remove isSneaking`, player);
                }
            }*/
            if (c(hitEntity)) {
                let player = hitEntity;
                if (player?.hasTag("kingCrimson.epitaph")) {
                    _this.runCommand(`tag @s add kingCrimson.avoid`, player);
                }
            }
        })
        _this.listenForEvent("minecraft:itemStartCharge", (eventData) => {
            let { itemStack, source } = eventData;
            if (c(source)) {
                let item = itemStack;
                let player = source;
                if (item.typeId === "stand:skill2") {
                    if (!player?.isSneaking) {
                        if (!player?.hasTag("kingCrimson.timeErase")) {
                            _this.runCommand(`tag @s add kingCrimson.timeErase`, player)
                        }
                        if (player?.hasTag("kingCrimson.timeErase")) {
                            _this.runCommand(`tag @s remove kingCrimson.timeErase`, player)
                        }
                    } else {
                        if (!player?.hasTag("kingCrimson.epitaph")) {
                            _this.runCommand(`tag @s add kingCrimson.epitaph`, player)
                        }
                        if (player?.hasTag("kingCrimson.epitaph")) {
                            _this.runCommand(`tag @s remove kingCrimson.epitaph`, player)
                        }
                    }
                }
            }
        })
    }
    
    APattack(data) {
        let player = this.standMaster;
        player?.addTag("stand.can_ap")
        if (!player?.hasTag("stand.APattack") && this.mental_value.var > 0 && this.ap_cd.var >= 0 && this.ap_cd.var < data.cooldown * 20) {
            this.ap_cd.var += 1;
        }
        if (player?.hasTag("stand.APattack") && this.ap_cd.var >= data.cooldown * 20 && this.mental_value.var > 0) {
            this.ap_cd2.var += 1;
            this.removeMentalValue(data.mental);
            this.broadcastAnim("stand->ap");
            if (this.ap_cd2.var == 2) {
                let entity = this.getEntitiesFromViewVector(player, data.range + 1).filter(entity => entity.hasComponent("minecraft:health") && entity.id.split(":")[0] != "kingcrimson")[0];
                if (entity !== undefined) {
                    this.runCommands([
                        `damage @s[type=!ender_dragon,type=!player] 0 suicide`,
                        `damage @s[m=adventure,type=player] 0 suicide`,
                        `damage @s[m=survival,type=player] 0 suicide`,
                        `damage @s[type=ender_dragon] 0 suicide entity @a[name="${player?.name}"]`,
                        `effect @s slowness 2 255 true`,
                        `tag @s remove isAttackedByAP`,
                        `playsound kingCrimson.attack @a ~ ~ ~`
                    ], entity);
                    
                    let test = new mc.MolangVariableMap();
                    let entity_pos = "xyz".split("").map(xyz => {
                        return entity.getHeadLocation()[xyz] - (xyz === "y") * 0.3
                    })
                    entity.dimension.spawnParticle("effect:blood",new Vec3(...entity_pos).Position(),test)
                    
                    let health = entity.getComponent("minecraft:health");
                    health.setCurrent(health.current - data.damage);
                }
                player?.removeTag("stand.APattack");
                this.ap_cd2.var = 0;
                this.ap_cd.var = 0;
            }
        } else if (player?.hasTag("stand.APattack") && this.ap_cd.var < data.cooldown * 20) {
            player?.removeTag("stand.APattack");
        }
        if (!player?.hasTag("stand.APattack")) {
            if (this.ap_cd.var == 3) {
                this.removeAnim("stand->ap");
            }
        }
    }
    
    epitaph(data) {
        let player = this.standMaster;
        
        if (player?.hasTag("kingCrimson.timeErase")) {
            player?.removeTag("kingCrimson.epitaph");
            // player?.removeTag("kc.epitaph_cooldown");
        }
        
        if (!player?.hasTag("kingCrimson.timeErase") && !player?.hasTag("kc.epitaph_cooldown") && player?.hasTag("kingCrimson.epitaph") && this.mental_value.var > 0 && this.epitaph_tick.var > 0 && this.epitaph_tick.var <= data.duration * 20) {
            this.epitaph_tick.var -= 1;
        }
        if (player?.hasTag("kc.epitaph_cooldown") && this.mental_value.var > 0 && this.epitaph_tick.var >= 0 && this.epitaph_tick.var < data.duration * 20) {
            this.epitaph_tick.var += 1;
            if (this.epitaph_tick.var < data.duration * 20) {
                player?.removeTag("kingCrimson.epitaph")
            }
        }
        if (this.epitaph_tick.var >= data.duration * 20) {
            player?.removeTag("kc.epitaph_cooldown");
        }
        if (!player?.hasTag("kingCrimson.timeErase") && !player?.hasTag("kc.epitaph_cooldown") && player?.hasTag("kingCrimson.epitaph") && this.epitaph_tick.var > 0 && this.mental_value.var > 0) {
            let health = player?.getComponent("minecraft:health");
            // this.runCommand(`say a`, player)
            this.epitaph_tick2.var += 1;
            if (this.epitaph_tick2.var == 1) {
                this.mental_timer.var = 1;
                this.broadcastAnim("king_crimson->epitaph");
                this.LoadingUI2(this.UI.kingCrimson.epitaph);
                this.playerHealth.var = health.current;
                this.runCommand(`playsound kingCrimson.epitaph @s ~ ~ ~`, player);
                this.MentalPanelUI();
            }
            if (this.epitaph_tick2.var > 1) {
                health.setCurrent(100);
                player?.addEffect(mc.MinecraftEffectTypes["instantHealth"], 19980, 255, false);
                
                if (player?.hasTag("isAttackedByFist")) {
                    player?.addTag("kingCrimson.avoid");
                }
                if (player?.hasTag("kingCrimson.avoid")) {
                    this.mental_timer.var = 1;
                    this.LoadingUI2(this.UI.kingCrimson.skip.default);
                    this.MentalPanelUI();
                    this.runCommands([
                        `spreadplayers ~ ~ 1 2 @s`,
                        `playsound kingCrimson.timeSkip2 @s ~ ~ ~`,
                        `playsound kingCrimson.timeSkip4 @a[tag=kingCrimson.epitaph] ~ ~ ~`
                    ], player);
                    player?.removeTag("kingCrimson.avoid");
                }
                if (this.epitaph_tick2.var % data.mental.interval == 0) {
                    this.removeMentalValue(data.mental.value);
                }
                /*
                if (this.epitaph_tick2.var % 2 == 0) {
                    this.runCommand(`tag @e[name=!"${player?.name}",r=2.3] add avoidTarget`, player);
                }
                this.getEntities({ excludeNames: [ player?.nameTag ], tags: [ "avoidTarget" ]}).filter(e => e.hasComponent("movement") || e.id == "minecraft:armor_stand" || e.id == "minecraft:boat" || e.id == "minecraft:minecart").forEach(entity => {
                    entity.removeTag("avoidTarget")
                })*/
                if (this.epitaph_tick2.var % 3 == 0) {
                    let projectiles = this.getEntities().filter(e => !e.hasTag("kingCrimson.epitaph") && !e.hasComponent("minecraft:movement") && e.id != "minecraft:armor_stand" && e.id != "minecraft:boat" && e.id != "minecraft:minecart" && e.id != "minecraft:item");
                    if (projectiles.length > 0) {
                        projectiles.forEach(projectile => {
                            if (projectile !== undefined) {
                                // this.runCommand(`tag @a[tag=kingCrimson.epitaph,r=2.5] add kingCrimson.avoid`, projectile);
                                let a = player?.location;
                                let b = projectile.location;
                                
                                if(new Vec3(b.x,b.y,b.z).Distance(a.x,a.y,a.z) <= 2.3 && player?.hasTag("kingCrimson.epitaph")) {
                                    player?.addTag("kingCrimson.avoid");
                                }
                            }
                        })
                    }
                }
            }

        } 
        if ((!player?.hasTag("kingCrimson.epitaph")) || (player?.hasTag("kingCrimson.epitaph") && this.epitaph_tick.var <= 0)) {
            if (this.epitaph_tick2.var > 0) {
                let health = player?.getComponent("minecraft:health");
                health.setCurrent(this.playerHealth.var);
                
                this.removeAnim("king_crimson->epitaph");
                this.runCommand(`effect @s instant_health 0 0 true`, player);
                this.epitaph_tick2.var = 0;
                
                player?.removeTag("kingCrimson.epitaph");
                player?.removeTag("kingCrimson.avoid");
                player?.addTag("kc.epitaph_cooldown");
            }
        }
    }
    
    timeErase(data) {
        let player = this.standMaster;
        let handItem = this.getHandItem(player);
        
        if (player?.hasTag("kingCrimson.timeErase") && this.erase_tick.var > 0 && this.erase_tick.var <= data.duration * 20) {
            this.erase_tick.var -= 1;
        }
        if (!player?.hasTag("kingCrimson.timeErase") && this.erase_tick.var >= 0 && this.erase_tick.var < data.duration * 20) {
            this.erase_tick.var += 1;
        }
        /*
        if (handItem.typeId == "stand:skill2" && player?.isSneaking && !player?.hasTag("isSneaking") && !player?.hasTag("kingCrimson.timeErase") && !player?.hasTag("kc.erase_cooldown") && this.erase_tick.var > 0) {
            player?.addTag("isSneaking");
            player?.addTag("kingCrimson.timeErase");
        }*/
        
        if (player?.hasTag("kingCrimson.timeErase") && !player?.hasTag("kc.erase_cooldown")) {
            this.erase_tick2.var += 1;
            if (this.erase_tick2.var == 1) {
                this.mental_timer.var = 1;
                this.LoadingUI2(this.UI.kingCrimson.skip.default);
                this.MentalPanelUI();
                this.broadcastAnim("king_crimson->time_skip");
                this.effect("saveCurrent");
                this.runCommand(`playsound kingCrimson.timeSkip1 @s ~ ~ ~`, player);
                let health = player?.getComponent("minecraft:health");
                this.playerHealth.var = health.current;
                player?.removeTag("kingCrimson.epitaph")
                /*
                for (let gamemode of [ "adventure", "creative", "survival" ]) {
                    this.runCommand(`tag @s[m=${gamemode}] add pgm:${gamemode}`, player);
                }*/
            }
            if (this.erase_tick2.var == 2) {
                // this.runCommand(`gamemode spectator @s`, player);
                this.broadcastAnim("king_crimson->player_invisibility");
                this.runCommand(`fog @s push fog:king_crimson king_crimson_fog`, player);
                player?.addTag("isSneaking2")
                // player?.addEffect(mc.MinecraftEffectTypes["darkness"], 19980, 1, false);
                /*
                let inventory = player?.getComponent("inventory").container;
                for (let slot of range(0,inventory.size)) {
                    let item = inventory.getItem(slot);
                    if (item !== undefined && item.typeId == "stand:skill1") {
                        player?.selectedSlot = slot;
                    }
                }*/
            }
            if (this.erase_tick2.var == 5) {
                this.runCommand(`playsound kingCrimson.timeErase @s ~ ~ ~`, player);
            }
            if (this.erase_tick2.var > 0) {
                let health = player?.getComponent("minecraft:health");
                if (health.current > 99) {
                    health.setCurrent(health.current + 1);
                }
                player?.addEffect(mc.MinecraftEffectTypes["speed"], 40, 2, false);
                // let movement = player?.getComponent("minecraft:movement");
                // movement.setCurrent(0.15);
                
            }
            if (this.erase_tick2.var % data.mental.interval == 0) {
                this.removeMentalValue(data.mental.value);
            }
            
            if (this.erase_tick2.var % 3 == 0) {
                this.effect("createNew");
            }
            let entities = this.getEntities({excludeTypes:["item"]}).filter(e => e !== player && !e.hasTag("kingCrimson.timeErase"));
            if (entities.length > 0) {
                entities.forEach(entity => {
                    /*
                    if (entity.hasComponent("minecraft:movement")) {
                        let movement = entity.getComponent("minecraft:movement");
                        movement.setCurrent(0.1)
                    }
                    */
                    let entity_pos = entity.location;
                    
                    let angleX = this.scores(entity,"angleX");
                    let angleY = this.scores(entity,"angleY");
                    
                    let velocityX = this.scores(entity,"velocityX");
                    let velocityY = this.scores(entity,"velocityY");
                    let velocityZ = this.scores(entity,"velocityZ");
                    
                    let posX_inF = this.scores(entity,"posX_inF");
                    let posY_inF = this.scores(entity,"posY_inF");
                    let posZ_inF = this.scores(entity,"posZ_inF");
                    
                    if (this.erase_tick2.var == 1) {
                        angleX.var = entity.getRotation().x.toFixed(2) * 100;
                        angleY.var = entity.getRotation().y.toFixed(2) * 100;
                        
                        velocityX.var = entity.getVelocity().x.toFixed(2) * 100;
                        velocityY.var = entity.getVelocity().y.toFixed(2) * 100;
                        velocityZ.var = entity.getVelocity().z.toFixed(2) * 100;
                    }
                    if (this.erase_tick2.var > 1) {
                        let entity_pxyz = "xyz".split("").map(xyz => {
                            return entity?.location[xyz];
                        })
                        let entity_vxyz = [
                            Mathf.clamp(velocityX.var/100,-1,1),
                            entity.getViewDirection().y,
                            Mathf.clamp(velocityZ.var/100,-1,1)
                        ];
                        let block = entity.dimension.getBlockFromRay(new Vec3(...entity_pxyz).Position(),new Vec3(...entity_vxyz).Vector(),{includeLiquidBlocks:false,includePassableBlocks:true,maxDistance:5});
                        let pos_inFd = block !== null && block !== undefined ? Math.round(Mathf.clamp(new Vec3(...entity_pxyz).Distance(block.location.x,block.location.y,block.location.z) - 1,0,3)) : 3;
                        if (this.erase_tick2.var % 4 == 0) {
                            // console.error(Math.round(Mathf.clamp(new Vec3(...entity_pxyz).Distance(block.location.x,block.location.y,block.location.z) - 1,0,3)))
                        }
                        entity.setRotation(angleX.var/100,angleY.var/100)
                        if (entity.typeId === "minecraft:player") {
                            for (let effect of [ "slowness->2", "blindness->20", "miningFatigue->10", "weakness->1" ]) {
                                let name = mc.MinecraftEffectTypes[effect.split("->")[0]];
                                let amplifier = parseInt(effect.split("->")[1]);
                                entity.addEffect(name, 20, amplifier, false);
                            }
                        } else {
                            entity.addEffect(mc.MinecraftEffectTypes["slowness"], 20, 4, false);
                            entity.applyImpulse(new Vec3(Mathf.clamp(velocityX.var,-0.043,0.043),0-!entity.hasComponent("minecraft:can_fly")*0.37,Mathf.clamp(velocityZ.var,-0.043,0.043)).Vector());
                        }

                        posX_inF.var = (Math.round(entity_pos.x) + Mathf.clamp(velocityX.var,-pos_inFd,pos_inFd)) * 100;
                        posZ_inF.var = (Math.round(entity_pos.z) + Mathf.clamp(velocityZ.var,-pos_inFd,pos_inFd)) * 100;
                        
                        let test = new mc.MolangVariableMap();
                        if (entity.hasComponent("minecraft:movement") || (!entity.hasComponent("minecraft:movement") && entity.getVelocity().x != 0 && entity.getVelocity().y != 0 && entity.getVelocity().z != 0)) {
                            entity.dimension.spawnParticle("kingcrimson:erase_effect",new Vec3(posX_inF.var/100,entity_pos.y,posZ_inF.var/100).Position(),test)
                        }
                    }
                    
                    
                })
            }
            
            /*
            if (this.erase_tick2.var > 3 && player?.isSneaking) {
                player?.removeTag("isSneaking");
                player?.removeTag("isSneaking2");
                player?.removeTag("kingCrimson.timeErase");
                player?.teleport({
                    x: player?.location.x,
                    y: player?.location.y,
                    z: player?.location.z
                },player?.dimension,player?.rotation.x,player?.rotation.y)
            }*/
        }
        if (player?.hasTag("kingCrimson.timeErase") && this.erase_tick.var <= 0) {
            player?.removeTag("kingCrimson.timeErase");
        }
        if ((!player?.hasTag("kingCrimson.timeErase") || (player?.hasTag("kingCrimson.timeErase") && this.erase_tick.var <= 0))) {
            if (this.erase_tick2.var > 0 && !player?.hasTag("kc.erase_cooldown")) {
                player?.addTag("isSneaking");
                player?.removeTag("kingCrimson.timeErase");
                this.mental_timer.var = 1;
                this.LoadingUI2(this.UI.kingCrimson.skip.default);
                this.MentalPanelUI();
                this.runCommand(`fog @s remove king_crimson_fog`, player);
                this.runCommand(`playsound kingCrimson.timeSkip2 @s ~ ~ ~`, player);
                this.runCommand(`effect @s speed 0 0 true`, player);
                // let movement = player.getComponent("minecraft:movement");
                // movement.resetToDefaultValue();
                // this.runCommand(`stopsound @s kingCrimson.timeErase`, player);
                /*
                player?.getTags().filter(tag => tag.startsWith("pgm:")).forEach(playerGameMode => {
                    this.runCommand(`gamemode ${playerGameMode.replaceAll("pgm:","")} @s`, player);
                    player?.removeTag(playerGameMode);
                })*/
                if (!player?.hasTag("stand.quick_fist")) {
                    this.removeAnim("king_crimson->time_skip");
                }

                
                this.erase_tick2.var = 0;
                this.effect("remove","clear");
                player?.addTag("kc.erase_cooldown")
            }
        }
        if (player?.hasTag("kc.erase_cooldown")) {
            player?.removeTag("kingCrimson.timeErase")
            if (this.erase_tick2.var < data.cooldown * 20) {
                this.erase_tick2.var += 1;
                if (this.erase_tick2.var == 2) {
                    let health = player?.getComponent("minecraft:health");
                    health.setCurrent(this.playerHealth.var)
                }
                if (this.erase_tick2.var == 3 || this.erase_tick2.var == 4) {
                    if (this.erase_tick2.var == 3) {
                        this.effect("remove","setCurrent")
                    }
                
                    let entities = this.getEntities({excludeTypes:["item"]}).filter(e => e !== player && !e.hasTag("kingCrimson.timeErase"));
                    if (entities.length > 0) {
                        entities.forEach(entity => {
                            let entity_pos = entity.location;
                            let posX_inF = this.scores(entity,"posX_inF");
                            let posY_inF = this.scores(entity,"posY_inF");
                            let posZ_inF = this.scores(entity,"posZ_inF");
                            let velocityX = this.scores(entity,"velocityX");
                            let velocityY = this.scores(entity,"velocityY");
                            let velocityZ = this.scores(entity,"velocityZ");
                            
                            if (this.erase_tick.var == 3) {
                                entity.teleport({
                                    x: posX_inF.var / 100,
                                    y: entity_pos.y,
                                    z: posZ_inF.var / 100
                                },entity.dimension,entity.getRotation().x,entity.getRotation().y);
                            }
                            
                            if (this.erase_tick2.var == 4 && !entity.hasComponent("minecraft:movement")) {
                                entity.applyImpulse(new Vec3(velocityX.var/100,velocityY.var/100,velocityZ.var/100).Vector())
                            }
                            
                            if (entity.typeId === "minecraft:player") {
                                for (let effect of [ "slowness", "blindness", "mining_fatigue", "weakness" ]) {
                                    this.runCommand(`effect @s ${effect} 0 0 true`, entity);
                                }
                            } else {
                                entity.addEffect(mc.MinecraftEffectTypes["slowness"], 20, 10, false);
                            } 
                        })
                    }
                }
            }
            if (this.erase_tick2.var >= data.cooldown * 20) {
                player?.removeTag("kc.erase_cooldown");
                this.erase_tick2.var = 0;
            }
        }
    }
    /*
    timeErase(data) {
        let tag = "绯红之王时间删除";
        let player = this.standMaster;
        
        if (player?.hasTag("player?.isSkippingTime") && this.skip_duration > 0) {
            this.playerScores.add(`skip_tick`,`scores={skip_tick=0..10}`,1);
            this.runCommand(`tag @s[scores={skip_tick=10..}] add kingCrimson.erase`, player);
            
            this.testfor(player, `scores={skip_tick=1}`, () => {
                this.effect("saveCurrent");
                this.broadcastAnim("king_crimson->time_skip");
            })
            this.testfor(player, `tag=!kingCrimson.skip,scores={skip_tick=4}`, () => {
                let playerCurrentHealth = `playerHealth_inNotSkippingTime: ${Math.round(this.health.current).toString()}`
                player?.addTag(playerCurrentHealth);
                player?.removeTag("kingCrimson.epitaph");
                player?.removeTag("kingCrimson.epitaph2");
                let fake_player = player?.dimension.spawnEntity("kingcrimson:fake_player", new Vec3(player?.location.x, player?.location.y, player?.location.z).Position());
                fake_player?.nameTag = player?.name;
                fake_player?.setRotation(-player?.rotation.x, player?.rotation.y);
                this.runCommands([
                    `playsound kingCrimson.timeSkip1 @s ~ ~ ~`,
                    `tag @e[type=!player,r=23] add entity.inSkippingTime`,
                    `fog @s push stand:crimson_fog king_crimson_fog`,
                    `tag @s add kingCrimson.skip`
                ], player);
                this.broadcastAnim("king_crimson->player_invisibility");
                this.LoadingUI(this.UI.kingCrimson.skip.on);
            });
            this.testfor(player, `tag=kingCrimson.erase`, () => {
                player?.removeTag("kingCrimson.epitaph");
                player?.removeTag("kingCrimson.epitaph2");
                this.playerScores.remove("skip_duration", `scores={skip_duration=1..${data.duration * 20}}`, 1);
                this.health.setCurrent(Math.min(this.health.current * 10, Math.round(Mathf.random(2000, 3000))));
                this.LoadingUI(this.UI.kingCrimson.redScreen);
                this.effect("createNew");
                this.playerScores.add(`skip_tick2`,`scores={skip_tick2=0..10}`,1)
                this.runCommands([
                    `playsound kingCrimson.timeErase @s[scores={skip_tick2=8}] ~ ~ ~`,
                    "tag @s[scores={skip_duration=0}] remove player?.isSkippingTime"
                ], player);
                let setEntitiesTarget = this.setEntitiesTargetInSkippingTime(
                    {
                        type: "kingcrimson:fake_player"
                    },
                    {
                        excludeTypes: [ "kingcrimson:fake_player", "item" ],
                        families: [ "monster" ],
                        tags: [ "entity.inSkippingTime" ]
                    },
                    {
                        excludeTypes: [ "kingcrimson:fake_player", "item" ],
                        families: [ "mob" ],
                        tags: [ "entity.inSkippingTime" ]
                    }
                )
            });
        }
        if (!player?.hasTag("player?.isSkippingTime")) {
            this.testfor(player, `tag=kingCrimson.skip`, () => {
                this.LoadingUI(this.UI.kingCrimson.skip.off);
                this.movement.resetToDefaultValue();
                this.removeAnim("king_crimson->time_skip",`scores={ap_cooldown=0..60}`);
                this.runCommands([
                    `event entity @e[name="${player?.name}"] despawn`,
                    `playsound kingCrimson.timeSkip2 @s ~ ~ ~`,
                    `playsound kingCrimson.timeSkip4 @a[tag=kingCrimson.skip] ~ ~ ~`,
                    `tag @e[type=!player] remove entity.inSkippingTime`,
                    `tag @s remove kingCrimson.skip`,
                    `tag @s[tag=kingCrimson.erase] remove kingCrimson.erase`,
                    `fog @s remove king_crimson_fog`
                ], player);
                player?.getTags().filter(tags => tags.includes("playerHealth_inNotSkippingTime")).forEach(playerDefaultHealth => {
                    let health = parseInt(playerDefaultHealth.split(": ")[1]);
                    this.health.setCurrent(health);
                    if (player?.hasTag(playerDefaultHealth)) {
                        player?.removeTag(playerDefaultHealth);
                    }
                });
                this.playerScores.set(`skip_tick`,0);
                this.playerScores.set(`skip_tick2`,0);
            });
            if (!player?.isSneaking) {
                this.playerScores.add("skip_duration", `scores={skip_duration=0..${data.duration * 20 - 1}}`, 1);
            }
            this.effect("remove");
        }
    }
    */
    
    effect(mode,type) {
        let player = this.standMaster;
        let effectList = {
            effect1: new Array("levitation", "wither", "slowness", "blindness", "poison", "fatal_poison", "hunger", "nausea"),
            effect2: new Array("mining_fatigue", "invisibility", "weakness", "resistance", "fire_resistance", "conduit_power", "instant_health"),
            effect3: new Array("jump_boost", "slow_falling")
        }
        let saveEffect = (effect) => {
            effectList[effect].forEach(value => {
                let getEffect = () => {
                    if (value.includes("_")) {
                        return this.standMaster.getEffect(mc.MinecraftEffectTypes[value.toLowerCase().replace(/_(\w)/g, (L) => L.toUpperCase()).replace(/_/g, "")]);
                    } else {
                        return this.standMaster.getEffect(mc.MinecraftEffectTypes[value.toLowerCase()]);
                    }
                }
                if (getEffect() !== undefined) {
                    player?.addTag(`effect:${value}-${Math.round(this.getTimer(getEffect().duration))}-${getEffect().amplifier}`);
                }
            });
        }
        let addEffect = (effect,duration,amplifier) => {
            effectList[effect].forEach(value => {
                if (value.includes("_")) {
                    return this.standMaster.addEffect(mc.MinecraftEffectTypes[value.toLowerCase().replace(/_(\w)/g, (L) => L.toUpperCase()).replace(/_/g, "")],duration*20,amplifier,false);
                } else {
                    return this.standMaster.addEffect(mc.MinecraftEffectTypes[value.toLowerCase()],duration*20,amplifier,false);
                }
            });
        }
        switch (mode) {
            case "saveCurrent":
                saveEffect("effect1");
                saveEffect("effect2");
                saveEffect("effect3");
                break;
            case "createNew":
                effectList.effect1.forEach(value => {
                    this.runCommand(`effect @s ${value} 0 0 true`, player)
                });
                addEffect("effect2",9999,255)
                addEffect("effect3",9999,2)
                break;
            case "remove":
                switch (type) {
                    case "clear":
                        effectList.effect2.forEach(value => {
                            this.runCommand(`effect @s ${value} 0 0 true`, player)
                        });
                        effectList.effect3.forEach(value => {
                            this.runCommand(`effect @s ${value} 0 0 true`, player)
                        });
                        break;
                    case "setCurrent":
                        player?.getTags().filter(tags => tags.includes("effect:")).forEach(effects =>{
                            let effect = {
                                name: effects.replace(/effect:/g, "").split("-")[0],
                                duration: parseInt(effects.replace(/effect:/g, "").split("-")[1]),
                                amplifier: parseInt(effects.replace(/effect:/g, "").split("-")[2])
                            }
                            this.runCommand(`effect @s ${effect.name} ${effect.duration} ${effect.amplifier}`, player);
                            if (player?.hasTag(effects)) {
                                player?.removeTag(effects)
                            }
                        })
                        break;
                }
                break;
        }
    }
    
}

