import * as mc from "@minecraft/server";
import * as mcui from "@minecraft/server-ui";
import Vec3 from "../../modules/Vec3.js";
import Mathf from "../../modules/Mathf.js";
import range from "../../modules/arrayRange.js";
import registerSystem from "../../register/registerSystem.js";

import "./default/settings_menu.js";

import standIndex from "./list/standIndex.js";

// import "./list/kingCrimson/_main_.js";
// import "./list/goldExperience/_main_.js";


class StandScript extends registerSystem {
    constructor() {
        super();
        this.stand = /stand:skill1/;
        this.listenForEvent("minecraft:beforeWatchdogTerminate", (eventData) => {
            eventData.cancel = true;
        })
        
        Object.keys(standIndex).forEach(stand => {
            return standIndex[stand].eventData(this);
        })
        this.tick(() => this.stand_ticking(),1);
        
        this.stand_ticking = () => {
            Object.keys(standIndex).forEach(stand => {
                let players = this.getPlayers({tags:[`standMaster.${stand}`]/*,excludeTags:["ui_using"]*/});
                if (players.length > 0) {
                    players.forEach(player => {
                        return new standIndex[stand](player);
                    })
                }
            })
        };
        
        
        
        this.listenForEvent("minecraft:scriptEventReceive", (eventData) => {
            let { id, initiator, message, sourceBlock, sourceEntity, sourceType } = eventData;
            if (id === "set:mental_value") {
                let command = (position, target = message.split(" ")[1]??="@s", value = parseInt(message.split(" ")[0])) => {
                    let pos = "xyz".split("").map(xyz => {
                        return position[xyz];
                    }).join(" ");
                    return `execute positioned ${pos} run tag ${target} add mentalValue_${value}`
                }
                if (sourceEntity !== undefined && sourceEntity.typeId === "minecraft:player") {
                    let player = sourceEntity;
                    this.runCommand(command(player?.location), player)
                }
                if (sourceBlock !== undefined) {
                    let block = sourceBlock;
                    this.runCommand(command(block?.location), this.getDimension("overworld"))
                }
            }
        });
        /*
        this.listenForEvent("minecraft:itemUseOn", (eventData) => {
            let { source, item } = eventData;
            if (item.typeId === "minecraft:diamond_sword") {
                let lc = this.getLc(source);
                source.applyKnockback(lc.x,lc.z,2,source.getViewDirection().y)
            }
        });
        */
        /*
        this.tick(() => {
            this.getPlayers().forEach(player => {
                let item = this.getHandItem(player);
                if (item.typeId === "minecraft:diamond_sword") {
                    let block = player?.getBlockFromViewDirection();
                    if (block !== undefined && block !== null) {
                        let sign = block.getComponent("minecraft:sign");
                        console.error(sign.getText());
                    }
                }
            })
        },15);
        this.listenForEvent("minecraft:entityHit", (eventData) => {
            let { entity, hitEntity } = eventData;
            let item = this.getHandItem(entity);
            if (item?.typeId === "minecraft:diamond_sword") {
                hitEntity?.setOnFire(10)
            }
        })*/
        /*
        this.tick(() => {
            this.getPlayers().forEach(player => {
                let item = this.getHandItem(player);
                if (item?.typeId === "weapon:flying_knife") {
                    let enchantments = item?.getComponent("minecraft:enchantments").enchantments;
                    enchantments.addEnchantment(new mc.Enchantment({id:"knockback"},100));
                }
            })
        },1),*/
        this.listenForEvent("custom:playerJoin", (eventData) => this.playerJoin(eventData));
        this.listenForEvent("minecraft:chat", (eventData) => this.chat(eventData));
        this.listenForEvent("minecraft:worldInitialize", (eventData) => this.worldInitialize(eventData));
        this.listenForEvent("minecraft:itemStartCharge", (eventData) => this.stand_startsAttack(eventData));
        this.listenForEvent("minecraft:itemStopCharge", (eventData) => this.stand_stopsAttack(eventData));
        this.listenForEvent("minecraft:itemStartCharge", (eventData) => this.stand_disc(eventData));
        this.listenForEvent("minecraft:entityDie", (eventData) => this.entityDeath(eventData));
        this.listenForEvent("minecraft:itemStopCharge", (eventData) => this.foodSystem(eventData));
        this.listenForEvent("minecraft:projectileHit", (eventData) => {
            let { source, projectile } = eventData;
            if (projectile.typeId === "weapon:flying_knife") {
                projectile.addTag("isHit")
            }
        });
        this.listenForEvent("minecraft:entitySpawn", (eventData) => this.scoresCreate(eventData));
        this.tick(() => this.update(),1);
        this.tick(() => {
            this.getPlayers().forEach(player => {
                let mental_value = this.GlobalVar(player,"mental_value");
                let mental_recovery = this.GlobalVar(mc.world,"mental.can_recovery");
                mental_value.var += mental_recovery.var ? 1 : 0;
            })
        },200);
        /*
        this.tick(() => {
            // Green Day!
            this.getPlayers().forEach(player => {
                if (player?.getVelocity().y < 0) {
                    player?.applyDamage(Math.abs(Math.round(player?.getVelocity().y * 10) + 1))
                }
            })
        },1);*/
        this.tick(() => {
            let players = this.getPlayers({tags:["pick_up.knife"]});
            if (players.length > 0) {
                players.forEach(player => {
                    let inventory = player?.getComponent("inventory")?.container;
                    let itemStack = new mc.ItemStack(mc.ItemTypes.get("weapon:flying_knife"));
                    inventory?.addItem(itemStack);
                    this.runCommand(`kill @e[type=weapon:flying_knife,tag=isHit,c=1,r=2]`, player)
                    player?.removeTag("pick_up.knife")
                })
            }
        },1)
        this.tick(() => {
            let items = this.getEntities({type: "item"}).filter(item => item.getComponent("minecraft:item").itemStack.typeId.split(":")[0] == "stand");
            if (items.length > 0) {
                items.forEach(item => {
                    item.kill();
                })
            }
            // this.scoresCreate();
            
            this.getPlayers().forEach(player => {
                let timer = this.GlobalVar(player,"mental_timer");
                timer.var += 1;
                if (timer.var > 3) {
                    timer.var = 0;
                }
                
                this.runCommand(`tag @s[hasitem={item=jojo:stand_settings,quantity=0}] add stand.no_settings`, player)
                
                let check = player?.getTags().join("|");
                let inventory = player?.getComponent("inventory").container;
                if (!check?.includes("standMaster.")) {
                    for (let slot of range(0,inventory.size - 1)) {
                        let item = inventory.getItem(slot);
                        if (item !== undefined) {
                            if (item.typeId.split(":")[0] == "stand") {
                                this.runCommand(`clear @s ${item.typeId} 0 999`, player);
                            }
                        }
                    }
                }
                
            })
        },10)
    }
    
    
    update() {
        let players = this.getPlayers();
        if (players.length > 0) {
            players.forEach(player => {
                let health = player?.getComponent("minecraft:health");
                
                player?.removeTag("standMaster.empty");
                let reborn_tick = this.GlobalVar(player,"reborn_tick")
                let mental_value = this.GlobalVar(player,"mental_value")
                let mental_timer = this.GlobalVar(player,"mental_timer");
                
                mental_value.var = Mathf.clamp(mental_value.var,0,70);
                
                if (player?.hasTag("stand.no_settings")) {
                    // console.error("hello world")
                    let inventory = player?.getComponent("inventory").container;
                    let itemStack = new mc.ItemStack(mc.ItemTypes.get("jojo:stand_settings"));
                    itemStack.keepOnDeath = true,
                    itemStack.lockMode = "slot",
                    // itemStack.nameTag = "hello world",
                    inventory?.setItem(8,itemStack)
                    player?.removeTag("stand.no_settings");
                }
                let tags = player?.getTags().filter(tag => tag.split("_")[0] == "mentalValue");
                if (tags?.length > 0) {
                    tags?.forEach(tag => {
                        mental_value.var = parseInt(tag.split("_")[1]);
                        
                        player?.removeTag(tag)
                    })
                }
                if (mental_value.var <= 0 && !player?.hasTag("player.no_mental")) {
                    mental_timer.var = 0;
                    player?.addTag("player.no_mental");
                } else {
                    player?.removeTag("player.no_mental");
                }
                if (player?.hasTag("player.no_mental")) {
                    for (let effect of [ "hunger->20", "miningFatigue->10", "slowness->3", "weakness->10" ]) {
                        let name = mc.MinecraftEffectTypes[effect.split("->")[0]];
                        let amplifier = parseInt(effect.split("->")[1]);
                        player?.addEffect(name, 20, amplifier, false);
                    }
                }
                if (player?.hasTag("standMaster.death") && health.current > 0 && player?.getEffect(mc.MinecraftEffectTypes["healthBoost"]) !== undefined) {
                    health.setCurrent(100);
                    player?.removeTag("standMaster.death");
                }
                /*
                if (!player?.getTags().join("|").includes("standMaster.")) {
                    player?.removeTag("stand.can_ap")
                    let color = {
                        x: "§b",
                        y: "§a",
                        z: "§c"
                    }
                    let xyz = (key) => {
                        return "txyz".split("").map(p => {
                            if (p == "t") {
                                return "§l" + key + "§r: "
                            } else {
                                return color[p] + player[key][p].toFixed(2) + "§r";
                            }
                        }).join(" ")
                    }
                    let xy = (key) => {
                        return "txy".split("").map(p => {
                            if (p == "t") {
                                return "§l" + key + "§r: "
                            } else {
                                return color[p] + player[key][p].toFixed(2) + "§r";
                            }
                        }).join(" ")
                    }
                    let debug_text = [
                        `§e[debug]§r\n`,
                        `${xyz("velocity")}\n`,
                        `${xyz("viewDirection")}\n`,
                        `${xyz("location")}\n`,
                        `${xy("rotation")}\n`,
                        `${xyz("headLocation")}\n`,
                        `§lfps§r: §b${this.GlobalVar(player,"fps").var}§r`
                    ]
                    let list = [
                        " ",
                        this.GlobalVar(player,"line_wrap").var,
                        ...debug_text,
                        this.GlobalVar(player,"line_wrap").var,
                        " ",
                        "\n\n\n",
                        " ",
                        "\n\n\n",
                        " ",
                        "\n\n\n",
                        " "
                    ]
                    let text = list.join("");
                    // player?.onScreenDisplay.setActionBar(text)
                }*/
                /*
                this.getEntities({type:["minecraft:pig"]}).forEach(entity => {
                    let pig_pos = entity.location;
                    let player_pos = player?.location;
                    
                    let t = new mc.Location(pig_pos.x,pig_pos.y,pig_pos.z);
                    if (entity.typeId === "minecraft:pig" && t.isNear(new mc.Location(player_pos.x,player_pos.y,player_pos.z),2)) {
                        this.runCommand(`say ${entity.typeId}`, entity);
                    }
                })*/
                
                
                /*
                if (handItem.typeId === "minecraft:diamond_sword") {
                    let lc = this.getLc(player);
                    player?.teleport({
                        x: player?.location.x + lc.x,
                        y: player?.location.y + lc.y,
                        z: player?.location.z + lc.z
                    }, player?.dimension, player?.rotation.x, player?.rotation.y)
                }*/
                
                // if (this.stand.exec(handItem.typeId) && player?.isSneaking) {
                    // player?.addTag("stand.APmode");
                    // this.runCommand(`effect @s weakness 9999 255 true`, player);
                // }
                // if (player?.hasTag("stand.APmode")) {
                    // this.runCommand(`effect @s weakness 0 0 true`, player);
                    // player?.removeTag("stand.APmode")
                // }
                
                
                
                
            })
        }
        
    }
    
    settings(eventData) {
        let { source, itemStack } = eventData;
        if (source.typeId === "minecraft:player") {
            let player = source;
            let item = itemStack;
            if (!player?.isSneaking && item.typeId == "jojo:stand_settings") {
                player?.addTag("ui.stand_settings.menu");
            }
        }
    }
    
    stand_disc(eventData) {
        let item = eventData.itemStack;
        let player = eventData.source;
        let check = player?.getTags();
        let includes = check.join("|").includes("standMaster.");
        if (item.typeId != "disc:empty" && /disc:\w+/.exec(item.typeId) && !includes) {
            let stand_name = item.typeId.replace(/disc:/g,"").toLowerCase().replace(/_(\w)/g, (L) => L.toUpperCase()).replace(/_/g, "");
            player?.addTag("standMaster." + stand_name);
            player?.addTag("stand.fist_isCooldown");
            player?.addTag("kc.epitaph_cooldown");
            player?.addEffect(mc.MinecraftEffectTypes["regeneration"], 100, 10, false);
            this.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 disc:empty`, player);
        }
        if (item.typeId == "disc:empty" && includes) {
            check.filter(tags => tags.includes("standMaster.")).forEach(stand_tag => {
                let disc_name = "disc:" + stand_tag.replace(/standMaster./g,"").split("").map(value => {
                    if (value == value.toUpperCase()) {
                        return "_" + value.toLowerCase();
                    } else {
                        return value;
                    }
                }).join().replace(/,/g,"");
                this.runCommands([
                    `replaceitem entity @s slot.weapon.mainhand 0 ${disc_name}`,
                    `effect @s health_boost 0 0 true`
                ], player);
                player?.removeTag(stand_tag);
                for (let i of range(1,4)) {
                    this.GlobalVar(player,"stand_skill" + i).var = 0;
                    this.GlobalVar(player,"stand_skill" + i + "_2").var = 0;
                }
            })
        }
    }

    stand_startsAttack(eventData) {
        let c = (target) => {
            return target.typeId === "minecraft:player" && target.getTags().join("|").includes("standMaster.");
        }
        let { itemStack, source } = eventData;
        
        if (c(source)) {
            let item = itemStack;
            let player = source;
            if (item.typeId === "stand:skill1") {
                if (player?.isSneaking && player?.hasTag("stand.can_ap")) {
                    this.runCommand(`tag @s add stand.APattack`, player)
                } else {
                    this.runCommand(`tag @s add stand.quick_fist`, player)
                }
            }
        }
    }
    stand_stopsAttack(eventData) {
        let c = (target) => {
            return target.typeId === "minecraft:player" && target.getTags().join("|").includes("standMaster.");
        }
        let { itemStack, source } = eventData;
        
        if (c(source)) {
            let item = itemStack;
            let player = source;
            if (item.typeId === "stand:skill1") {
                this.runCommand(`tag @s remove stand.quick_fist`, player)
            }
        }
    }
    
    entityDeath(eventData) {
        let entity = eventData.deadEntity;
        for (let tag of ["isAttackedByFist","isHitByFist"]) {
            if (entity.hasTag(tag)) {
                let health = entity.getComponent("minecraft:health");
                entity.removeTag(tag);
            }
        }
        if (entity.typeId == "minecraft:player" && entity.getTags().join("|").includes("standMaster.")) {
            this.GlobalVar(entity,"mental_value").var = 70;
            this.GlobalVar(entity,"reborn_tick").var = 0;
            
            // entity.addTag("standMaster.death");
            // entity.removeTag("stand.quick_fist");
            // entity.removeTag("stand.APattack");
            
            this.runCommands([
                `tag @s add standMaster.death`,
                `tag @s remove stand.quick_fist`,
                `tag @s remove stand.APattack`
            ], entity)
        }
    }
    worldInitialize(eventData) {
        let def = new mc.DynamicPropertiesDefinition();
        let def2 = new mc.DynamicPropertiesDefinition();
        let list1 = [
            "tick",
            "fps_tick",
            "fps",
            "delta_time",
            
            "stand_skill1",
            "stand_skill2",
            "stand_skill3",
            "stand_skill4",
            "stand_skill1_2",
            "stand_skill2_2",
            "stand_skill3_2",
            "stand_skill4_2",
            
            
            
            "reborn_tick",
            
            "mental_value",
            "mental_reply",
            "mental_timer",
            
            "playerHealth",
            
            "stand_fp_offset"
        ].forEach(value => {
            def.defineNumber(value);
        })
        
        let list2 = [
            "clicked_blockFace->30",
            "goldExperience.entity->30",
            "playerGameMode->30",
            "line_wrap->64"
        ].forEach(value => {
            let name = value.split("->")[0];
            let length = parseInt(value.split("->")[1]);
            def.defineString(name,length);
        })
        
        let list3 = [
            "barrage_camera_shake",
            "mental.is_infinite",
            "mental.can_recovery"
        ].forEach(value => {
            def.defineBoolean(value);
        })
        
        let list4 = [
            "mental.is_infinite",
            "mental.can_recovery",
            "clear_tnts",
            "clear_items"
        ].forEach(value => {
            def2.defineBoolean(value);
        })
        
        let list5 = [
            "stand_range_limit"
        ].forEach(value => {
            def2.defineNumber(value);
        })
        
        eventData.propertyRegistry.registerEntityTypeDynamicProperties(def, mc.MinecraftEntityTypes.player);
        eventData.propertyRegistry.registerWorldDynamicProperties(def2);
    }
    
    playerJoin(eventData) {
        
        let player = eventData.player;
        this.scoresCreate(eventData);
        player?.addTag("stand.fist_isCooldown");
        player?.addTag("kc.epitaph_cooldown");
        player?.removeTag("kingCrimson.epitaph");
        player?.removeTag("kingCrimson.epitaph2");
        player?.removeTag("kingCrimson.avoid");
        player?.removeTag("kingCrimson.skip");
        player?.removeTag("kingCrimson.erase");
        player?.removeTag("ui_using");
        
        player?.removeTag("standMaster.death");
        
        let translate = [
            "ui.stand_settings.3.addon.golden_wind.description.start",
            "ui.stand_settings.3.addon.golden_wind.description.1",
            "ui.stand_settings.3.addon.golden_wind.description.2",
            "ui.stand_settings.3.addon.golden_wind.description.3",
            "ui.stand_settings.3.addon.golden_wind.description.4",
            "ui.stand_settings.3.addon.golden_wind.description.5",
            "ui.stand_settings.3.addon.golden_wind.description.6",
            "ui.stand_settings.3.addon.golden_wind.description.7",
            "ui.stand_settings.3.addon.golden_wind.description.end"
        ];
        let text = translate.map(t => {
            return "<translate>" + t
        })
        this.GlobalVar(player,"reborn_tick").var??=0;
        this.GlobalVar(player,"mental_value").var??=70;
        this.GlobalVar(player,"mental_reply").var??=0;
        this.GlobalVar(player,"mental_timer").var??=0;
        this.GlobalVar(player,"barrage_camera_shake").var??=true;
        this.GlobalVar(player,"stand_fp_offset").var??=35;
        this.GlobalVar(mc.world,"stand_range_limit").var??=0;
        this.GlobalVar(mc.world,"clear_tnts").var??=true;
        this.GlobalVar(mc.world,"clear_items").var??=true;
        this.GlobalVar(mc.world,"mental.is_infinite").var??=false;
        this.GlobalVar(mc.world,"mental.can_recovery").var??=true;
        this.GlobalVar(player,"goldExperience.entity").var??="goldexperience:tree";
        
        let message = this.listMessage(...text);
        this.runCommand(`tellraw @s ${message}`, player);
        
        let list = [
            "tick",
            "fps_tick",
            "fps",
            "delta_time"
        ];
        list.forEach(value => {
            this.GlobalVar(player,value).var = 0;
        })
        let list2 = [
            "stand_skill1",
            "stand_skill1_2",
            "stand_skill2",
            "stand_skill2_2",
            "stand_skill3",
            "stand_skill3_2",
            "stand_skill4",
            "stand_skill4_2"
        ];
        list2.forEach(value => {
            this.GlobalVar(player,value).var = 0;
        })
        
        let array = [];
        for (let i of range(0,63)) {
            array.push("\n")
        }
        this.GlobalVar(player,"line_wrap").var = array.join("")
        
        // this.runCommand(`playanimation @s animation.stand.first_person_offset.variable.${stand_fp_offset.var} default 1`, player);
    }
    
    chat(eventData) {
        let player = eventData.sender;
        let message = eventData.message;
        if (message.startsWith("#D4C")) {
            let dimension = this.getDimension(message.split(":")[1]);
            player?.teleport({
                x: 0,
                y: 0,
                z: 0
            }, dimension, player?.rotation.x, player?.rotation.y);
        }
    }
    
    scoresCreate(eventData) {/*
        let entity = eventData.entity;
        let globalScores = this.scoresStatic();
        globalScores.create(
            entity,
            "stand_range->stand_range",
            "angleX->rx",
            "angleY->ry",
            "velocityX->vx",
            "velocityY->vy",
            "velocityZ->vz",
            "posX_inF->pfx",
            "posY_inF->pfy",
            "posZ_inF->pfz"
        )
        globalScores.setdisplay(
            "stand_range/belowname"
        )*/
    }
    
    listMessage(...texts) {
        let i = 0;
        let array = [...texts];
        
        while (i <= array.length) {
            array.splice(i, 0, "\n");
            i += 2;
        }
        let object = array.slice(1).slice(0, -1).map(v => {
            if (v === "\n") {
                return {
                    text: v
                }
            } else if (v.startsWith("<text>")) {
                return {
                    text: v.replaceAll("<text>","")
                }
            } else if (v.startsWith("<translate>")) {
                return {
                    translate: v.replaceAll("<translate>","")
                }
            }
        });
        
        let textJson = JSON.stringify({
            rawtext: [
                ...object
            ]
        })
        return textJson;
    }
    
    foodSystem(eventData) {
        let item = eventData.itemStack;
        let entity = eventData.source;
        let mc_food = [
    		"apple->4",
    		"enchanted_golden_apple->4",
    		"baked_potato->5",
    		"beef->3",
    		"beetroot->1",
    		"beetroot_soup->6",
    		"bread->5",
    		"carrot->3",
    		"chicken->2",
    		"chorus_fruit->4",
    		"clownfish->1",
    		"cooked_beef->8",
    		"cooked_chicken->6",
    		"cooked_fish->5",
    		"cooked_porkchop->8",
    		"cooked_rabbit->5",
    		"cooked_salmon->6",
    		"cookie->2",
    		"dried_kelp->1",
    		"fish->2",
    		"golden_apple->4",
    		"golden_carrot->6",
    		"melon_slice->2",
    		"mushroom_stew->6",
    		"cooked_mutton->6",
    		"mutton->2",
    		"poisonous_potato->2",
    		"porkchop->3",
    		"potato->1",
    		"pufferfish->1",
    		"pumpkin_pie->8",
    		"rabbit->3",
    		"rabbit_stew->10",
    		"rotten_flesh->4",
    		"salmon->2",
    		"spider_eye->2",
    		"suspicious_stew->6",
    		"sweet_berries->2"
    	].map(v => {
    	    return "minecraft:" + v;
    	});
    	if (entity.typeId === "minecraft:player") {
    	    let player = entity;
    	    let mental_value = this.GlobalVar(player,"mental_value");
    	    let mental_timer = this.GlobalVar(player,"mental_timer");
        	if (item.typeId.split(":")[0] === "minecraft") {
            	mc_food.forEach(value => {
            	    let food = {
                	    id: value.split("->")[0],
                	    nutrition: parseInt(value.split("->")[1])
            	    }
            	    if (item.typeId === food.id) {
                        let { useDuration } = eventData;
                        let { nutrition } = food;
            	        if (useDuration <= 0) {
            	            mental_value.var += nutrition;
            	            mental_timer.var = 0;
            	        }
            	        // this.runCommand(`say d:${useDuration} n:${nutrition}`, player);
            	    }
            	})
        	}/*
        	if (item.hasComponent("minecraft:food") && item.typeId.split(":")[0] !== "minecraft") {
    	        let cFood = item.getComponent("minecraft:food");
    	        let useDuration = eventData.useDuration;
    	        foodData(useDuration,cFood.nutrition);
    	        // this.runCommand(`say d:${useDuration} n:${cFood.nutrition}`, player);
    	    }*/
	    }
    }

}

new StandScript();
